package edu.mermet.tp8;


import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import javax.swing.*;

import edu.mermet.tp8.fenetres.*;


/**
 *
 * @author brunomermet
 */
public class Application extends JFrame implements PropertyChangeListener {

    private JFrame frame;

    private JInternalFrame conversion;
    private JInternalFrame texte;
    private JInternalFrame diaporama;
    private JInternalFrame boutons;
    private JInternalFrame monnaie;

    private Action actionAfficherConversion;
    private Action actionAfficherTexte;
    private Action actionAfficherDiaporama;
    private Action actionAfficherBoutons;
    private Action actionAfficherMonnaie;

    private Action actionAfficherCommentFaire;
    private Action actionAfficherConfigMenu;

    private JMenu menuFichier;
    private JMenuItem quitter;

    private JMenu menuLangue;

    private JMenu menuApplication;
    private JMenuItem itemConversion;
    private JMenuItem itemTexte;
    private JMenuItem itemDiaporama;
    private JMenuItem itemBoutons;
    private JMenuItem itemMonnaie;


    private ButtonGroup groupLangue;

    private PropertyChangeSupport pcs;


    public Application() {
        super("multi-fenêtres");
        this.setContentPane(new JDesktopPane());

        // ****** Barre de menu ******
        JMenuBar barre = new JMenuBar();
        // ------ menu Fichier ------
        menuFichier = new JMenu("Fichier");
        menuFichier.setMnemonic(KeyEvent.VK_F);
        quitter = new JMenuItem("Quitter");
        quitter.addActionListener (new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent aev) {
                System.exit(0);
            }
        });
        quitter.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Q, InputEvent.CTRL_DOWN_MASK));
        menuFichier.add(quitter);
        barre.add(menuFichier);
        this.setJMenuBar(barre);
        // ------ menu Applications ------
        menuApplication = new JMenu("Applications");
        menuApplication.setMnemonic(KeyEvent.VK_A);
        actionAfficherConversion = new ActionAfficherConversion();
        itemConversion = new JMenuItem(actionAfficherConversion);
        menuApplication.add(itemConversion);
        actionAfficherTexte = new ActionAfficherTexte();
        itemTexte = new JMenuItem(actionAfficherTexte);
        menuApplication.add(itemTexte);
        actionAfficherDiaporama = new ActionAfficherDiaporama();
        itemDiaporama = new JMenuItem(actionAfficherDiaporama);
        menuApplication.add(itemDiaporama);
        actionAfficherBoutons = new ActionAfficherBoutons();
        itemBoutons = new JMenuItem(actionAfficherBoutons);
        menuApplication.add(itemBoutons);

        actionAfficherMonnaie = new ActionAfficherMonnaie();
        itemMonnaie = new JMenuItem(actionAfficherMonnaie);
        menuApplication.add(itemMonnaie);
        barre.add(menuApplication);

        menuLangue = new JMenu("Langue");

        Action francais = new Langue("FR");
        Action anglais = new Langue("EN");

        groupLangue = new ButtonGroup();

        JRadioButtonMenuItem menuItemFrancais = new JRadioButtonMenuItem(francais);
        menuItemFrancais.setSelected(true);
        groupLangue.add(menuItemFrancais);
        menuLangue.add(menuItemFrancais);

        JRadioButtonMenuItem menuItemAnglais = new JRadioButtonMenuItem(anglais);
        groupLangue.add(menuItemAnglais);
        menuLangue.add(menuItemAnglais);

        barre.add(menuLangue);

        // ------ menu Aide ------
        JMenu menuAide = new JMenu("Aide");
        menuAide.setMnemonic(KeyEvent.VK_H);
        actionAfficherCommentFaire = new ActionAfficherCommentFaire();
        JMenuItem itemCommentFaire = new JMenuItem(actionAfficherCommentFaire);
        menuAide.add(itemCommentFaire);
        actionAfficherConfigMenu = new ActionAfficherConfigMenu();
        JMenuItem itemConfigMenu = new JMenuItem(actionAfficherConfigMenu);
        menuAide.add(itemConfigMenu);
        barre.add(menuAide);
        // ****** Fin barre de menu ******

        // ****** Création des fenêtres ******
        // ------ fenêtre conversion ------
        conversion = new FenetreConversion(actionAfficherConversion);
        this.add(conversion);
        // ------ fenêtre texte ------
        texte = new FenetreTexte(actionAfficherTexte);
        this.add(texte);
        // ------ fenêtre diaporama ------
        diaporama = new FenetreDiaporama(actionAfficherDiaporama);
        this.add(diaporama);
        // ------ fenêtre boutons ------
        boutons = new FenetreBoutons(this,actionAfficherBoutons);
        this.add(boutons);

        monnaie = new FenetreConversionMonnaie(actionAfficherMonnaie);
        this.add(monnaie);


        // ****** Fin création fenêtres ******
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        setSize(600,300);
        this.setLocationRelativeTo(null);
        setVisible(true);

        this.pcs = new PropertyChangeSupport(this);
        addPropertyChangeListener((PropertyChangeListener)conversion);
        addPropertyChangeListener((PropertyChangeListener)texte);
        addPropertyChangeListener((PropertyChangeListener)diaporama);
        addPropertyChangeListener((PropertyChangeListener)boutons);
        addPropertyChangeListener((PropertyChangeListener)monnaie);
        addPropertyChangeListener((PropertyChangeListener)this);

        frame = this;

    }

    class Langue extends AbstractAction{
        public Langue(String text){
        super(text);
        putValue("langue", text);
    }

    public void actionPerformed(ActionEvent e) {
        PropertyChangeEvent event = new PropertyChangeEvent(this,"langue",null,getValue("langue"));
        Application.this.firePropertyChangeEvent(event);
    }
}

    public class ActionAfficherMonnaie extends AbstractAction {
        public ActionAfficherMonnaie() {
            super("Conversion monnaie");
            putValue(Action.ACCELERATOR_KEY,KeyStroke.getKeyStroke(KeyEvent.VK_M, InputEvent.CTRL_DOWN_MASK));
            putValue(Action.MNEMONIC_KEY,KeyEvent.VK_M);
        }

        @Override
        public void actionPerformed(ActionEvent ae) {
            monnaie.setVisible(true);
            enableMonnaie(false);
        }
    }

    public void propertyChange(PropertyChangeEvent evt) {

        switch (evt.getNewValue().toString()){
            case "FR":
                setTitle("multi-fenêtres");
                menuFichier.setText("Fichier");
                quitter.setText("Quitter");
                menuLangue.setText("Langue");
                menuApplication.setText("Applications");
                itemBoutons.setText("Boutons");
                itemConversion.setText("Conversion celsius/Farenheit");
                itemDiaporama.setText("Diaporama");
                itemTexte.setText("Saisie de texte");
                break;
            case "EN":
                setTitle("multi-windows");
                menuFichier.setText("File");
                quitter.setText("Quit");
                menuLangue.setText("Language");
                menuApplication.setText("Applications");
                itemBoutons.setText("Button");
                itemConversion.setText("Conversion celsius/Farenheit");
                itemDiaporama.setText("Slideshow");
                itemTexte.setText("Text entry");
                break;
        }
    }

    public void addPropertyChangeListener(PropertyChangeListener listener) {
        this.pcs.addPropertyChangeListener(listener);
    }

    public void removePropertyChangeListener(PropertyChangeListener listener) {
        this.pcs.removePropertyChangeListener(listener);
    }

    public void firePropertyChangeEvent(PropertyChangeEvent evt) {
        this.pcs.firePropertyChange(evt);
    }

    private class ActionAfficherBoutons extends AbstractAction {
        public ActionAfficherBoutons() {
            super("Boutons");
            putValue(Action.ACCELERATOR_KEY,KeyStroke.getKeyStroke(KeyEvent.VK_B, InputEvent.CTRL_DOWN_MASK));
            putValue(Action.MNEMONIC_KEY,KeyEvent.VK_B);
        }

        @Override
        public void actionPerformed(ActionEvent ae) {
            boutons.setVisible(true);
            enableBoutons(false);
        }
    }

    private class ActionAfficherDiaporama extends AbstractAction {
        public ActionAfficherDiaporama() {
            super("Diaporama");
            putValue(Action.ACCELERATOR_KEY,KeyStroke.getKeyStroke(KeyEvent.VK_D, InputEvent.CTRL_DOWN_MASK));
            putValue(Action.MNEMONIC_KEY,KeyEvent.VK_D);
        }

        @Override
        public void actionPerformed(ActionEvent ae) {
            diaporama.setVisible(true);
            enableDiaporama(false);
        }
    }

    private class ActionAfficherTexte extends AbstractAction {
        public ActionAfficherTexte() {
            super("Saisie de texte");
            putValue(Action.ACCELERATOR_KEY,KeyStroke.getKeyStroke(KeyEvent.VK_T, InputEvent.CTRL_DOWN_MASK));
            putValue(Action.MNEMONIC_KEY,KeyEvent.VK_T);
        }

        @Override
        public void actionPerformed(ActionEvent ae) {
            texte.setVisible(true);
            enableTexte(false);
        }
    }

    public class ActionAfficherConversion extends AbstractAction {
        public ActionAfficherConversion() {
            super("Conversion Celsius/Farenheit");
            putValue(Action.ACCELERATOR_KEY,KeyStroke.getKeyStroke(KeyEvent.VK_C, InputEvent.CTRL_DOWN_MASK));
            putValue(Action.MNEMONIC_KEY,KeyEvent.VK_C);
        }

        @Override
        public void actionPerformed(ActionEvent ae) {
            conversion.setVisible(true);
            enableConversion(false);
        }
    }

    public class ActionAfficherCommentFaire extends AbstractAction {
        public ActionAfficherCommentFaire() {
            super("Comment Faire ?");
            putValue(Action.ACCELERATOR_KEY,KeyStroke.getKeyStroke(KeyEvent.VK_F, InputEvent.CTRL_DOWN_MASK));
        }

        @Override
        public void actionPerformed(ActionEvent ae) {
            new CommentFaire();
        }
    }

    public class ActionAfficherConfigMenu extends AbstractAction {
        public ActionAfficherConfigMenu() {
            super("Configuration des menus");
            putValue(Action.ACCELERATOR_KEY,KeyStroke.getKeyStroke(KeyEvent.VK_G, InputEvent.CTRL_DOWN_MASK));
        }

        @Override
        public void actionPerformed(ActionEvent ae) {
            new ConfigurationMenu(frame);
        }
    }

    public void enableConversion(boolean b) {
        actionAfficherConversion.setEnabled(b);
    }

    public void enableTexte(boolean b) {
        actionAfficherTexte.setEnabled(b);
    }

    public void enableDiaporama(boolean b) {
        actionAfficherDiaporama.setEnabled(b);
    }

    public void enableBoutons(boolean b) {
        actionAfficherBoutons.setEnabled(b);
    }

    public void enableMonnaie(boolean b) {
        actionAfficherMonnaie.setEnabled(b);
    }


    public void enableCommentFaire(boolean b){
        actionAfficherCommentFaire.setEnabled(b);
    }

    public void enableConfigMenu(boolean b){
        actionAfficherConfigMenu.setEnabled(b);
    }

    public Action getActionAfficherConversion() {
        return actionAfficherConversion;
    }

    public Action getActionAfficherTexte() {
        return actionAfficherTexte;
    }

    public Action getActionAfficherDiaporama() {
        return actionAfficherDiaporama;
    }


    public Action getActionAfficherMonnaie() {
        return actionAfficherMonnaie;
    }

    public static void main(String[] args) {
        SwingUtilities.invokeLater(Application::new);
    }


}
